'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var coffee = _interopDefault(require('coffeescript'));
var path = _interopDefault(require('path'));
var rollupPluginutils = require('rollup-pluginutils');
var fs = _interopDefault(require('fs'));
var acorn = require('acorn');
var estraverse = _interopDefault(require('estraverse'));
var escodegen = _interopDefault(require('escodegen'));

var formatError = function (source, filename, err) {
    var carets, col, first_column, first_line, l, last_column, last_line, lines;
    ({first_line, last_line, first_column, last_column} = err.location);
    if (last_line == null) {
        last_line = first_line;
    }
    lines = source.split('\n');
    l = lines[first_line];
    lines[first_line] = l.substring(0, first_column) + '\x1B[91m' + l.substring(first_column);
    l = lines[last_line];
    if (first_line === last_line) {
        col = last_column + 6;
    } else {
        col = last_column;
    }
    lines[last_line] = l.substring(0, col) + '\x1B[39m' + l.substring(col);
    lines = lines.slice(first_line, last_line + 1);
    carets = Array(first_column + 1).join(' ') + `\x1B[91m${ Array(last_column + 2 - first_column).join('^') }\x1B[39m`;
    lines.push(carets);
    return `${ filename }:${ first_line + 1 }:${ first_column + 1 }: ${ err.name }: ${ err.message }\n${ lines.join('\n') }\n`;
};

var sourceMap;
sourceMap = function (out) {
    if (out.v3SourceMap) {
        return JSON.parse(out.v3SourceMap);
    } else {
        return null;
    }
};
var coffee2 = function (opts = {}) {
    var filter;
    if (opts.bare == null) {
        opts.bare = true;
    }
    if (opts.sourceMap == null) {
        opts.sourceMap = true;
    }
    filter = rollupPluginutils.createFilter(opts.include, opts.exclude);
    return {
        name: 'coffee2',
        transform: function (code, id) {
            var err, out;
            if (!filter(id)) {
                return { code };
            }
            if (path.extname(id) !== '.coffee') {
                return { code };
            }
            try {
                out = coffee.compile(code, {
                    filename: id,
                    bare: opts.bare,
                    sourceMap: opts.sourceMap
                });
            } catch (error) {
                err = error;
                if (err.location != null) {
                    err.formattedMessage = formatError(code, id, err);
                }
                throw err;
            }
            return {
                code: out.js,
                map: sourceMap(out)
            };
        }
    };
};

function __async(g){return new Promise(function(s,j){function c(a,x){try{var r=g[x?"throw":"next"](a);}catch(e){j(e);return}r.done?s(r.value):Promise.resolve(r.value).then(c,d);}function d(e){c(e,1);}c();})}

var ASTToCode;
var getAST;
var rCFPath;
var ricffAST;
var _extends = Object.assign || function (target) {
        for (var i = 1; i < arguments.length; i++) {
            var source = arguments[i];
            for (var key in source) {
                if (Object.prototype.hasOwnProperty.call(source, key)) {
                    target[key] = source[key];
                }
            }
        }
        return target;
    };
rCFPath = filePath => {
    var fileObj, getFileObj;
    getFileObj = function (file) {
        return _extends({ exists: fs.existsSync(file) }, path.parse(file));
    };
    fileObj = getFileObj(filePath);
    if (!fileObj.exists) {
        fileObj = {
            coffee: function () {
                var _path;
                _path = path.format({
                    root: path.join(fileObj.root, fileObj.dir + '/'),
                    name: fileObj.base,
                    ext: '.coffee'
                });
                return {
                    path: _path,
                    obj: getFileObj(_path)
                };
            }(),
            js: function () {
                var _path;
                _path = path.format({
                    root: path.join(fileObj.root, fileObj.dir + '/'),
                    name: fileObj.base,
                    ext: '.js'
                });
                return {
                    path: _path,
                    obj: getFileObj(_path)
                };
            }()
        };
        if (fileObj.coffee.obj.exists) {
            return rCFPath(fileObj.coffee.path);
        }
        if (fileObj.js.obj.exists) {
            return rCFPath(fileObj.js.path);
        }
        return null;
    } else {
        if (fileObj.ext !== '') {
            return filePath;
        } else {
            return rCFPath(path.join(filePath, 'index.coffee'));
        }
    }
};
ricffAST = (ast, dirname = __dirname) => {
    return estraverse.replace(ast, {
        leave: function (currentNode, parentNode) {
            var fileParseObj, filePath;
            if (currentNode.type === 'ImportDeclaration') {
                fileParseObj = path.parse(currentNode.source.value);
                if (fileParseObj.root === '' && fileParseObj.dir === '') {
                    return currentNode;
                }
                filePath = rCFPath(path.join(dirname, currentNode.source.value));
                if (filePath != null) {
                    currentNode.source.value = filePath;
                }
            }
            return currentNode;
        }
    });
};
getAST = source => {
    var comments, tokens;
    comments = [];
    tokens = [];
    return acorn.parse(source, {
        sourceType: 'module',
        ranges: true,
        onComment: comments,
        Token: tokens
    });
};
ASTToCode = function (ast) {
    return escodegen.generate(ast);
};
exports.testFun = function () {
    return __async(function* () {
        return yield console.log('hello');
    }());
};

var sourceMap$1;
sourceMap$1 = function (out) {
    if (out.v3SourceMap) {
        return JSON.parse(out.v3SourceMap);
    } else {
        return null;
    }
};
var coffeepath = function (opts = {}) {
    var filter;
    filter = rollupPluginutils.createFilter(opts.include, opts.exclude);
    return {
        name: 'coffeepath',
        transform: function (code, id) {
            if (!filter(id)) {
                return { code };
            }
            if (path.extname(id) !== '.coffee') {
                return { code };
            }
            return {
                code: ASTToCode(ricffAST(getAST(code), path.dirname(id))),
                map: sourceMap$1(code)
            };
        }
    };
};

exports.coffee2 = coffee2;
exports.coffeepath = coffeepath;
